#include "cdaq_utils.h"

#include "cstatistics_utils.h"


cDaq_utils::cDaq_utils(QObject *parent) :
    QObject(parent)
{
    //load_engr_coefs();  //need to call from mainwindow() as it also has to give us a pointer to csql class

    cStats = new cStatistics_utils(this);

    //csql = new cSQL_Utils();  //DO THIS IN MAINWINDOW()
}


void cDaq_utils::load_engr_coefs()  //load engr coefs from data-base
{
    //get engr parms from sqlite DB

    QString tableName = "ENGR_PARMS";

    QString col_IDName = "PARM_ID";
    QString queryColName = "VALUE";


    try
    {
        QString row_IDValue = "SLOPE_PRESSURE";
        QString sVal = csql->getValue_colQuery_table(col_IDName, row_IDValue, queryColName, tableName);
        slope_press = sVal.toDouble();

        row_IDValue = "INTERCEPT_PRESSURE";
        sVal = csql->getValue_colQuery_table(col_IDName, row_IDValue, queryColName, tableName);
        intercept_press = sVal.toDouble();


        row_IDValue = "SLOPE_WEIGHT";
        sVal = csql->getValue_colQuery_table(col_IDName, row_IDValue, queryColName, tableName);
        slope_weight = sVal.toDouble();

        row_IDValue = "INTERCEPT_WEIGHT";
        sVal = csql->getValue_colQuery_table(col_IDName, row_IDValue, queryColName, tableName);
        intercept_weight = sVal.toDouble();



        row_IDValue = "SLOPE_01_WEIGHT";
        sVal = csql->getValue_colQuery_table(col_IDName, row_IDValue, queryColName, tableName);
        slope_01_weight = sVal.toDouble();

        row_IDValue = "INTERCEPT_01_WEIGHT";
        sVal = csql->getValue_colQuery_table(col_IDName, row_IDValue, queryColName, tableName);
        intercept_01_weight = sVal.toDouble();


        row_IDValue = "SLOPE_12_WEIGHT";
        sVal = csql->getValue_colQuery_table(col_IDName, row_IDValue, queryColName, tableName);
        slope_12_weight = sVal.toDouble();

        row_IDValue = "INTERCEPT_12_WEIGHT";
        sVal = csql->getValue_colQuery_table(col_IDName, row_IDValue, queryColName, tableName);
        intercept_12_weight = sVal.toDouble();


        row_IDValue = "SLOPE_23_WEIGHT";
        sVal = csql->getValue_colQuery_table(col_IDName, row_IDValue, queryColName, tableName);
        slope_23_weight = sVal.toDouble();

        row_IDValue = "INTERCEPT_23_WEIGHT";
        sVal = csql->getValue_colQuery_table(col_IDName, row_IDValue, queryColName, tableName);
        intercept_23_weight = sVal.toDouble();


        row_IDValue = "WEIGHT_THRESHOLD_01";
        sVal = csql->getValue_colQuery_table(col_IDName, row_IDValue, queryColName, tableName);
        weight_threshold_01 = sVal.toDouble();

        row_IDValue = "WEIGHT_THRESHOLD_12";
        sVal = csql->getValue_colQuery_table(col_IDName, row_IDValue, queryColName, tableName);
        weight_threshold_12 = sVal.toDouble();


        row_IDValue = "STABILITY_THRESHOLD_WEIGHT";
        sVal = csql->getValue_colQuery_table(col_IDName, row_IDValue, queryColName, tableName);
        stability_threshold_weight = sVal.toDouble();


        row_IDValue = "LOAD_TARE";
        sVal = csql->getValue_colQuery_table(col_IDName, row_IDValue, queryColName, tableName);
        sTareWeight = sVal;
        tare_weight = sVal.toDouble();




        tableName = "DATA";
        int lastLoadID = csql->getLast_rowID_in_table(tableName);

        loadNum = lastLoadID + 1;


    }
    catch(...)
    {

    }





}


void cDaq_utils::receive_data(QStringList strlist)
{
    //convert daq_msg to raw_data;
    parse_message(strlist);

    int cnt = strlist.count();
    if(cnt < 50)
    {
        return;
    }


//  parse_message(daq_msg);

    cnt = raw_data.count();
    int halfway = cnt /2 ;
    int third = cnt / 3;
    int twoThirds = 2*third;


    QList<double> first_half = raw_data.mid(0,halfway);
    QList<double> second_half = raw_data.mid(halfway+1,cnt-1);

    QList<double> first_third = raw_data.mid(0,third);
    QList<double> second_third = raw_data.mid(third+1,twoThirds);
    QList<double> last_third = raw_data.mid(twoThirds+1,cnt-1);

    QList<double> last_twoThirds = raw_data.mid(third+1, cnt-1);



    double mean_first_half = cStats->calc_mean(first_half);
    double mean_second_half = cStats->calc_mean(second_half);

    double mean_second_third = cStats->calc_mean(second_third);
    double mean_last_third = cStats->calc_mean(last_third);


    //convert to pressure

    mean_first_half = calc_pressure(mean_first_half);
    mean_second_half = calc_pressure(mean_second_half);

    mean_first_half = calc_weight(mean_first_half);
    mean_second_half = calc_weight(mean_second_half);


    mean_second_third = calc_pressure(mean_second_third);
    mean_last_third = calc_pressure(mean_last_third);

    mean_second_third = calc_weight(mean_second_third);
    mean_last_third = calc_weight(mean_last_third);


    double mean_raw_val = cStats->calc_mean(raw_data);


    mean_raw_val = cStats->calc_mean(last_twoThirds);


    stability = mean_second_half - mean_first_half;

    stability = mean_last_third - mean_second_third;


    if(stability < 0.0)
    {
        stability = stability * -1.0;
    }


    //stability = cStats->calc_stdev(raw_data);


    sStability = QString::number(stability,'f',3);

    calc_pressure(mean_raw_val);

    sPressure  = QString::number(pressure,'f',1);

    calc_weight(pressure);


//not here; do it in mainwindow
    //get area
    //calc yield


    if(bTareMode)
    {
        sLoadNum = "TARE";
        //bTareMode = false;  //not yet; do it in mainwindow's readDaq()

        tare_weight = -weight;

        sTareWeight = QString::number(tare_weight,'f',6);


        QString tableName = "ENGR_PARMS";

        int rowID = 25;
        QStringList colNames;
        colNames << "VALUE";

        QStringList columnValues;
        columnValues << sTareWeight;


        bool ok = csql->updateROW_table(tableName, rowID, colNames, columnValues);

        rowID++;

    }
    else
    {
        loadNum ++;
        sLoadNum = QString::number(loadNum);

        weight += tare_weight;
    }


    sWeight = QString::number(weight,'f',1);



    //UPDATE YIELD BASED ON WEIGHT
    double area = cgps->area_harvested;

    if(area > 0.0)
    {
        yield = weight / area;

        try
        {
            sYield = QString::number(yield,'f', 1);
        }
        catch(...)
        {

        }

    }
    else
    {
        yield = 0.0;
        sYield = "NAN";
    }


    //try saving here
    QString tableName = "DATA";
    QStringList tableColumnNames;
    QStringList  columnValues;

    //tableColumnNames << "LOAD_NUM" << "WEIGHT" << "FARM";
    //columnValues << sLoadNum << sWeight << "LAZY W";

    //int rowID = csql->insertROW_into_table(tableName, tableColumnNames, columnValues);


    QString msg = "dataReady";

    emit daq_ready(msg);

}

void cDaq_utils::test_daq()
{
    //receive_data(test_msg1);

   // receive_data(test_msg2);

}


QString cDaq_utils::get_dataRequest_string()
{
    //check pwm rates and modify weighOpRequest to suit requested rates

    return weighOpRequest;
}


bool cDaq_utils::id_is_in_list(int id)
{
    int cnt = raw_data_ids.count();
    bool found = false;

    for(int i=0;i<cnt;i++)
    {
        int cId = raw_data_ids[i];

        if(cId == id)
        {
            found = true;
            break;
        }
    }


    return found;
}

void cDaq_utils::extract_data(QStringList strlist)
{
    int cnt = strlist.count();

    raw_data.clear();
    raw_data_ids.clear();


    for(int i=0;i<cnt;i++)
    {
        QString str = strlist[i];

        if(str.contains("data"))
        {
            if(str.contains("data0:"))
            {

            }
            else
            {

                QStringList sLine_list = str.split(":");

                int count = sLine_list.count();

                if(count == 2)
                {
                    QString sName = sLine_list[0];

                    QStringList sList = sName.split("data");
                    count = sList.count();
                    if(count == 2)
                    {
                        try
                        {
                            QString sId = sList[1];
                            int id = sId.toInt();

                            if(id > data_below_to_ignore)
                            {
                                if(id > data_above_to_ignore)
                                {
                                    break;
                                }

                                //chk if id is already in list
                                bool bInList = id_is_in_list(id);

                                if(!bInList)
                                {
                                    QString sData = sLine_list[1];

                                    count = sData.count();

                                    if(count > 0)
                                    {

                                        double fdata = sData.toDouble();

                                        raw_data_ids.append(id);
                                        raw_data.append(fdata);
                                    }
                                    else
                                    {
                                        count++;
                                    }
                                }
                            }
                        }
                        catch(...)
                        {

                        }
                    }

                }

            }
        }
    }

}

//convert raw msg string to list<double)
void cDaq_utils::parse_message(QStringList strlist)
{
    //extract raw_data list from msg

    //load QList<double> raw_data

    //extract data into raw_data list and list of ids "raw_data_ids"
    this->extract_data(strlist);


    //now run statistics on it; mean stdev ect
    //find settling time ect

}


double cDaq_utils::calc_pressure(double val)
{
    double result = -99999.99;

    result = slope_press * val + intercept_press;

    pressure = result;

    return result;
}


double cDaq_utils::calc_weight(double val)
{
    double result = -99999.99;

    //1st estimate

    result = slope_weight * val + intercept_weight;


    if(result < weight_threshold_01)
    {
        result = slope_01_weight * val + intercept_01_weight;
    }
    else if(result < weight_threshold_12)
    {
        result = slope_12_weight * val + intercept_12_weight;
    }
    else
    {
        result = slope_23_weight * val + intercept_23_weight;
    }


    weight = result;

    sWeight = QString::number(weight,'f',1);

    return result;

}




