#include "cgps_utils.h"
#include "chaversine.h"

#include <QtCore>
#include <qmath.h>

#include <QString>
#include <QStringList>



cGPS_utils::cGPS_utils(QObject *parent) :
    QObject(parent)
{
}


void cGPS_utils::mark_start_GPS()
{
    start_lat_degs = lat_degs;
    start_lon_degs = lon_degs;

    last_lat_degs = lat_degs;
    last_lon_degs = lon_degs;

    sStartLat = sLat;
    sStartLon = sLon;

    sStartUTC = sUTC;


    dist = 0.0;
    total_dist = 0.0;
    sDist = "0.0";
    sArea_harvested = "0.0";

    gps_mark_status = true;


}


void cGPS_utils::mark_stop_GPS()
{
    gps_mark_status = false;


}

void cGPS_utils::recieve_GPS(QString message)
{
    //cHaversine cHav;

    try
    {
        //extract gps and update position
        extract_gps_msg(message);


        if(gps_mark_status == true)
        {
            //compute dist and add to running total dist traveled since MarkStart

            lat1 = last_lat_degs;
            lon1 = last_lon_degs;

            lat2 = lat_degs;  //set to our lat known position
            lon2 = lon_degs;

            //dist = getGPS_Dist_m(lat1, lon1, lat2, lon2);

            dist = cHav->gps_distance(lat1,lon1,lat2,lon2);

            total_dist += dist;

            sDist = QString::number(total_dist,'f', 2);

            //in acres
            area_harvested = total_dist * num_rows * row_width_ft / area_acre_sqft;

            sArea_harvested = QString::number(area_harvested,'f', 2);

            last_lat_degs = lat_degs;
            last_lon_degs = lon_degs;

            sStopUTC = sUTC;

        }

        //let mainwindow know gps has been updated so it can update display;
        //so need to emit a signal here
        emit gps_ready("ready");
    }
    catch(...)
    {
        int x = 5;
        int y = 5;

    }

}



//assumes both lat/lon 1 & lat/lon 2 are loaded
void cGPS_utils::calc_gps_dist()
{
    cHaversine cHav;

    //dist = getGPS_Dist_m(lat1, lon1, lat2, lon2);

    dist = cHav.gps_distance(lat1, lon1, lat2, lon2);
}

//from wiki link:  http://rosettacode.org/wiki/Haversine_formula

/*
class POI {
    private $latitude;
    private $longitude;
    public function __construct($latitude, $longitude) {
        $this->latitude = deg2rad($latitude);
        $this->longitude = deg2rad($longitude);
    }
    public function getLatitude(){ return $this->latitude};
    public function getLongitude(){ return $this->longitude};
    public function getDistanceInMetersTo(POI $other) {
        $radiusOfEarth = 6371000;// Earth's radius in meters.
        $diffLatitude = $other->getLatitude() - $this->latitude;
        $diffLongitude = $other->getLongitude() - $this->longitude;
        $a = sin($diffLatitude / 2) * sin($diffLatitude / 2) +
            cos($this->latitude) * cos($other->getLatitude()) *
            sin($diffLongitude / 2) * sin($diffLongitude / 2);
        $c = 2 * asin(sqrt($a));
        $distance = $radiusOfEarth * $c;
        return $distance;
    }
}
*/

//GPS Message RMC
/*
$GPRMC,123519,A,4807.038,N,01131.000,E,022.4,084.4,230394,003.1,W*6A

Where:
     RMC          Recommended Minimum sentence C
     123519       Fix taken at 12:35:19 UTC
     A            Status A=active or V=Void.
     4807.038,N   Latitude 48 deg 07.038' N
     01131.000,E  Longitude 11 deg 31.000' E
     022.4        Speed over the ground in knots
     084.4        Track angle in degrees True
     230394       Date - 23rd of March 1994
     003.1,W      Magnetic Variation
     *6A          The checksum data, always begins with *
*/


double cGPS_utils::gps_lat_conv2degs(QString sLat_gps)
{
    double latDegs = -99999.9;

    try
    {
        QString sLat_degs = sLat_gps.left(2);
        QString sLat_mins = sLat_gps.mid(2);


        int cnt = sLat_degs.length();


        double lat_mins = 0.0;

        if(cnt == 2)
        {
           cnt = sLat_mins.length();

           if(cnt >= 5)
           {
              latDegs = sLat_degs.toDouble();
              lat_mins = sLat_mins.toDouble();

              latDegs = latDegs + lat_mins/60.0;
           }
        }

    }
    catch(...)
    {

    }


    return latDegs;

}

double cGPS_utils::gps_lon_conv2degs(QString sLon_gps)
{
    double lonDegs = -99999.9;

    try
    {

        QString sLon_degs = sLon_gps.left(3);
        QString sLon_mins = sLon_gps.mid( 3);


        int cnt = sLon_degs.length();

        double lon_mins = 0.0;

        if(cnt == 3)
        {
           cnt = sLon_mins.length();

           if(cnt >= 5)
           {
              lonDegs = sLon_degs.toDouble();  //(float)$lon_degs;
              lon_mins = sLon_mins.toDouble();  //(float)$lon_mins;

              lonDegs = lonDegs + lon_mins/60.0;
           }
        }

    }
    catch(...)
    {

    }

    return lonDegs;

}



void cGPS_utils::extract_gps_msg(QString gps_msg)
{
    QStringList nmea_chunks = gps_msg.split(',');

    int cnt = nmea_chunks.count();

    if(cnt >= 9)
    {

        //$data = array();

        if(cnt > 10)
        {
           sUTC = nmea_chunks[1];
           sLat  = nmea_chunks[3];
           sLon  = nmea_chunks[5];
           sVel_knots  = nmea_chunks[7];
           sHeading_degs = nmea_chunks[8];
           sDate_gps = nmea_chunks[9];

           lat_degs  = gps_lat_conv2degs(sLat);
           lon_degs  = gps_lon_conv2degs(sLon);

           //$data = array("utc" => $utc, "lat" => $lat_degs, "lon" => $lon_gps, "vel_knots" => $vel_knots, "heading_degs" => $heading_degs);
           //data << utc
        }

        //return $data;  //directly loaded so not needed here
    }

}


double cGPS_utils::get_radius_earth_m(double lat_degs, double altitude_m)
{
     double lat_rads = degs2rads(lat_degs);

     double a = 6378000; // m (3963 mi) Equatorial radius (surface to center distance)
     double b = 6357000; // m (3950 mi) Polar radius (surface to center distance)
     double e = 0.081082; // Eccentricity

     double r_m = a * qSqrt(1.0 - e*e) / (1.0 - e*e * (qSin(lat_rads)*qSin(lat_rads)));

     r_m = r_m + altitude_m;

     return r_m;  //

}

double cGPS_utils::degs2rads(double x_degs)
{
    double x_rads = x_degs / 360.0 * 2.0 * 3.14159265;

    return x_rads;
}

double cGPS_utils::getGPS_Dist_m(double lat1_degs, double lon1_degs, double lat2_degs, double lon2_degs)
{
    double lat1_rads = degs2rads(lat1_degs);
    double lat2_rads = degs2rads(lat2_degs);
    double lon1_rads = degs2rads(lon1_degs);
    double lon2_rads = degs2rads(lon2_degs);

    //$radiusOfEarth = 6371000.0;// Earth's radius in meters.
    double radiusOfEarth = 6367000.0;// better approx for lat=30deg. Earth's radius in meters.

    double diffLatitude = lat2_rads - lat1_rads;
    double diffLongitude = lon2_rads - lon1_rads;

    double a = qSin(diffLatitude / 2) * qSin(diffLatitude / 2) +
             qCos(lat1_rads) * qCos(lat2_rads) *
             qSin(diffLongitude / 2) * qSin(diffLongitude / 2);

    double c = 2 * qAsin(sqrt(a));

    double distance_m = radiusOfEarth * c;

    return distance_m;

}

void cGPS_utils::test_gps()
{
    gps_mark_status = false;
    recieve_GPS(sGPS_msg1);

    mark_start_GPS();
    recieve_GPS(sGPS_msg2);

}


