#include "mainwindow.h"
#include "ui_mainwindow.h"


#include <QMessageBox>
#include <QStandardPaths>
#include <QString>
#include <QStringList>
#include <QThread>

#include <QFile>
#include <QTextStream>

#include <QStandardPaths>


#include "settingsdialog.h"

#include "cdialog_setup.h"

#include "test_sqlUtils.h"



class myHeaderView : public QHeaderView
{
public:
      myHeaderView():QHeaderView(Qt::Horizontal)
      {
    //setClickable(true);
    QPalette* palette = new QPalette();
    palette->setColor(QPalette::Button,Qt::cyan);
    setPalette(*palette);
      };
      ~myHeaderView(){};

};



MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);


    QStringList paths = QStandardPaths::standardLocations(QStandardPaths::DesktopLocation);
    QString desktopPath = paths[0];
    csv_filePath = desktopPath + "/weigh_data.csv";


    serial_gps = new QSerialPort(this);
    serial_daq = new QSerialPort(this);


    settings_daq = new SettingsDialog;
    settings_gps = new SettingsDialog;

    csql = new cSQL_Utils;

    csql->open_sqlite_dataBase();  //this opens default database


    cgps = new cGPS_utils(this);  //need to make connection for slot/signals
    cdaq = new cDaq_utils(this);
    cHav = new cHaversine(this);

    cgps->cHav = cHav;

    cdaq->csql = csql;

    cdaq->cgps = cgps;


    cdaq->load_engr_coefs();


    connect(serial_gps, SIGNAL(error(QSerialPort::SerialPortError)), this,
            SLOT(handleError_GPS(QSerialPort::SerialPortError)));

    connect(serial_daq, SIGNAL(error(QSerialPort::SerialPortError)), this,
            SLOT(handleError_DAQ(QSerialPort::SerialPortError)));


//! [2]
    connect(serial_gps, SIGNAL(readyRead()), this, SLOT(readData_gps()));
    connect(serial_daq, SIGNAL(readyRead()), this, SLOT(readData_daq()));


    connect(cgps, SIGNAL(gps_ready(QString)), this, SLOT(readGPS(QString)));

    connect(cdaq, SIGNAL(daq_ready(QString)), this, SLOT(readDAQ(QString)));



    QPalette* palette = new QPalette();
    palette->setColor(QPalette::Foreground,Qt::red);

    ui->frame->setPalette(*palette);
    ui->frame_2->setPalette(*palette);


    QString sDist = "0.0";
    QString sAcres = "0.0";
    QString sUTC = "now";

    load_gps_data(sDist, sAcres, sUTC);


    //init_database();

    load_settings_from_database();  //loads serial port PORT_NAME strings; and other serial port parms


    run_setup();



    //put time delay in before loading serial ports so our display has a chance to setup
    int msecs = 200;

    timer = new QTimer(this);
    timer->setInterval(msecs);
    timer->start();

    connect(timer, SIGNAL(timeout()), this, SLOT(load_serialPorts()));


}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::load_serialPorts()
{
    timer->stop();

    //auto connect  (note: load_settings_from_database() must run before these 2, as that's where port name is setup
    openSerialPort_GPS();
    openSerialPort_DAQ();

}


void MainWindow::run_setup()
{
    try
    {
        load_farmInfo_from_database();

        //launch dialog (leaves main window running)
        cDialog_Setup cdialog_setup;

        cdialog_setup.csql = csql;

        //load
        //sFarm = "ZZ TOP";
        //sClient = "JDW";
        //sField = "F5";

        cdialog_setup.sFarm = sFarm;
        cdialog_setup.sClient = sClient;
        cdialog_setup.sField = sField;
        cdialog_setup.sNum_rows = cgps->sNum_rows;
        cdialog_setup.sRow_width = cgps->sRow_width_in;

        cdialog_setup.sWindCondition = sWindCondition;


        //cdialog_setup.showMaximized();  //run as non-blocking
        cdialog_setup.setWindowState(Qt::WindowMaximized);
        cdialog_setup.exec();  //run as blocking dialog

        //getting info back is fine
        sFarm = cdialog_setup.sFarm;
        sClient = cdialog_setup.sClient;
        sField = cdialog_setup.sField;
        sNum_rows = cdialog_setup.sNum_rows;
        sRow_width = cdialog_setup.sRow_width;

        //NEED TO UPDATA cgps for num rows, row width

        cgps->sNum_rows=sNum_rows;
        cgps->num_rows = sNum_rows.toDouble();

        cgps->sRow_width_in = sRow_width;
        cgps->row_width_in = sRow_width.toDouble();
        cgps->row_width_ft = cgps->row_width_in / 12.0;



        sWindCondition = cdialog_setup.sWindCondition;

        cdaq->sWindCondition = sWindCondition;


        //update farm info
        QString tableName = "farmINFO";

        QStringList colNames;
        colNames <<  "ROW_WIDTH" << "NUM_ROWS";

        QStringList columnValues;

        columnValues << sRow_width << sNum_rows;


        //GET LAST ROW ID
        int cnt = csql->getLast_rowID_in_table(tableName);
        int rowID = cnt;


        bool ok = csql->updateROW_table(tableName, rowID, colNames, columnValues);


        bool y = ok;


        //    load_farmInfo_from_database();
    }
    catch(...)
    {}

}


void MainWindow::init_gps_display()  //run after connection
{
    QString sDist = "0.0";
    QString sAcres = "0.0";
    QString sUTC = "now";

    load_gps_data(sDist, sAcres, sUTC);

}

void MainWindow::init_daq_display()    //run after connection
{
    QString sLoadNum = "0";
    QString sFieldID = cdaq->sFieldID;;
    QString sYield = "0.0";
    QString sWeight = "0";
    QString sAcres = "0.0";

    int rowNum = 1;

    load_daq_rowData(rowNum, sLoadNum, sFieldID, sYield, sWeight, sAcres);




    sLoadNum = "";
    sFieldID = "";
    sYield = "";
    sWeight = "";
    sAcres = "";

    for(int i=2;i<11;i++)
    {
        rowNum = i;

        write_daq_tableWidget_item(sLoadNum, rowNum, 0);
//        write_daq_tableWidget_item(sFieldID, rowNum, 1 );
        write_daq_tableWidget_item(sYield, rowNum, 3 );
        write_daq_tableWidget_item(sWeight, rowNum, 2 );
        write_daq_tableWidget_item(sAcres, rowNum, 1 );

    }


}


//setup
void MainWindow::on_pushButton_clicked()
{
    run_setup();

}


//Mark Area Start
void MainWindow::on_pushButton_2_clicked()
{

    cgps->mark_start_GPS();


}

//Mark Area Stop
void MainWindow::on_pushButton_3_clicked()
{
    int numRows = 10;
    int numCols = 4;

    bubble_daq_tableWidget_rows_down(numRows, numCols);

    cgps->mark_stop_GPS();


    QString sLoadNum = "";
    QString sFieldID = "";
    QString sYield = "";
    QString sWeight = "";
    QString sAcres = "";

    int rowNum = 1;

    load_daq_rowData(rowNum, sLoadNum, sFieldID, sYield, sWeight, sAcres);


}

void MainWindow::send_command_message()
{



    QStringList strlist = full_data_str.split("\n");

    full_data_str.clear();



    //QString req_str = cdaq->get_dataRequest_string();
//Note: also have to change the capture routine to match these #samples/request in readData_daq()
    //default, 6 secs (3 we throw away, and 3secs we keep
    //QString req_str = "$,11,29,1,27,102,30,97,31,90,32,1000,33,1,18,10,8,10,1,10,2,10,9,10\n\r";


    QString req_str1_calm = "$,11,29,1,27,82,";
    QString req_str1_normal = "$,11,29,1,27,102,";
    QString req_str1_windy = "$,11,29,1,27,142,";


    //QString req_str2 = "30,97,31,90,";

    int pwm1 = cdaq->pwm1_firstStage_dutyCycle;
    int pwm2 = cdaq->pwm1_secondStage_dutyCycle;

    QString sPwm1 = QString::number(pwm1);
    QString sPwm2 = QString::number(pwm2);

    QString req_str2 = "30," + sPwm1 + ",31," + sPwm2 + ",";

    QString req_str3 = "32,1000,33,1,18,10,8,10,1,10,2,10,9,10\n\r";



    QString req_str = req_str1_normal + req_str2 + req_str3;

    if(sWindCondition.contains("CALM"))
    {
        //req_str = "$,11,29,1,27,82,30,97,31,90,32,1000,33,1,18,10,8,10,1,10,2,10,9,10\n\r";
        req_str = req_str1_calm + req_str2 + req_str3;
    }
    else if(sWindCondition.contains("WINDY"))
    {
        //req_str = "$,11,29,1,27,142,30,97,31,90,32,1000,33,1,18,10,8,10,1,10,2,10,9,10\n\r";
        req_str = req_str1_windy + req_str2 + req_str3;
    }





    QByteArray req_data = req_str.toLatin1();
    char* pSerial_msg = req_data.data();

    serial_daq->write(pSerial_msg);


}


//Run Weigh OP
void MainWindow::on_pushButton_4_clicked()
{
    cgps->mark_stop_GPS();  //ensure we've stopped GPS

    //orig
    //ui->pushButton_4->setStyleSheet("color: yellow; background-color:blue; border: 1px solid white");
    ui->pushButton_4->setStyleSheet("color: black; background-color:cyan; border: 1px solid black");

    this->send_command_message();
}


//TARE
void MainWindow::on_pushButton_5_clicked()
{

    cdaq->bTareMode = true;
    this->send_command_message();

}



//SLOTS
void MainWindow::readGPS(QString msg)
{
    try
    {

        int row = 1;
        int col = 2;

        //update display
        QString sDist = cgps->sDist;
        //sDist = "23";

        write_gps_tableWidget_item(sDist, row, col);



        col = 3;

        QString sArea = cgps->sArea_harvested;

        write_gps_tableWidget_item(sArea, row, col);


        col = 2;

        QString sUTC = cgps->sUTC;

    //    write_gps_tableWidget_item(sUTC, row, col);

        col = 0;
        write_gps_tableWidget_item(sFarm, row, col);

        col = 1;
        write_gps_tableWidget_item(sField, row, col);




        //col = 2;
        //write_gps_tableWidget_item(sField, row, col);


        ui->statusBar->showMessage(tr("GPS UTC: %1").arg(sUTC));

    }
    catch(...)
    {

    }

}


//call back from DAQ that means we can display the new data from the latest WeighOP
void MainWindow::readDAQ(QString msg)
{

    //chk stability parm
    double stab = cdaq->stability;
    double thres = cdaq->stability_threshold_weight;

    if(stab > thres)
    {
        QMessageBox msgBox;
        msgBox.setText("Stability exceeded, suggest re-weighing");
        msgBox.setInformativeText("Do you want to save this weight?");
        msgBox.setStandardButtons(QMessageBox::Save | QMessageBox::Discard);
        //msgBox.setStandardButtons( QMessageBox::Ok);

        int ret = msgBox.exec();
        switch(ret)
        {
            case QMessageBox::Save:
                break;
            case QMessageBox::Discard:
                //this->send_command_message();
                return;
            case QMessageBox::Ok:
                //this->send_command_message();
                return;
            default:
                break;

        }

    }



    //1st copy all data down 1 row
    //

    int numRows = 10;
    int numCols = 4;

    if(startUpState)
    {
        startUpState = false;
    }
    else
    {
        bubble_daq_tableWidget_rows_down(numRows, numCols);
    }

    //next update 1st row with the new data

    int row = 1;
    int col = 0;

    //update display
    QString sLoadNum = cdaq->sLoadNum;
    //sDist = "23";

    write_daq_tableWidget_item(sLoadNum, row, col);


    col = 1;
    QString sFieldID = cdaq->sFieldID;
//    write_daq_tableWidget_item(sFieldID, row, col);


    col = 3;
    QString sYield = cdaq->sYield;
    if(cdaq->bTareMode == true)
    {
        sYield = "";
    }
    write_daq_tableWidget_item(sYield, row, col);


    col = 2;
    QString sWeight = cdaq->sWeight;
    write_daq_tableWidget_item(sWeight, row, col);


    col = 1;
    QString sArea = cgps->sArea_harvested;
    write_daq_tableWidget_item(sArea, row, col);
    if(cdaq->bTareMode == true)
    {
        sArea  = "";
    }
    write_daq_tableWidget_item(sArea, row, col);


    //Last; insert all data into SQL table
    QStringList colNames;
    QStringList dataVals;

    colNames << "LOAD_NUM" << "CLIENT" << "FARM" << "FIELD" << "NUM_ROWS" << "ROW_WIDTH" << "YIELD" << "WEIGHT" << "AREA";
    colNames << "DISTANCE" << "STABILITY" << "PRESSURE" << "START_LAT" << "START_LON" << "STOP_LAT" << "STOP_LON" << "UTC_START" << "UTC_STOP";

    dataVals << cdaq->sLoadNum << sClient << sFarm << sField << cgps->sNum_rows << cgps->sRow_width_in << cdaq->sYield << cdaq->sWeight << cgps->sArea_harvested;
    dataVals << cgps->sDist << cdaq->sStability << cdaq->sPressure << cgps->sStartLat << cgps->sStartLon << cgps->sLat << cgps->sLon << cgps->sStartUTC << cgps->sStopUTC;


    //colNames.clear();
    //dataVals.clear();

    //colNames << "LOAD_NUM" << "CLIENT" << "FARM" << "FIELD" << "WEIGHT";
    //dataVals << cdaq->sLoadNum << sClient << sFarm << sField  << cdaq->sWeight;

    save_data_to_sql(colNames, dataVals);




    QFile file(csv_filePath);

    if(file.open(QFile::Append | QFile::Truncate))
    {
        QTextStream out(&file);

        QString sData;
        QString sHeader;

        int cnt = dataVals.count();

        if(first_csvWrite)
        {
            for(int i=0;i<cnt-1;i++)
            {
                 sHeader += colNames[i] + ",";
            }
            sHeader += colNames[cnt-1] + "\r\n";

            out << sHeader;

            first_csvWrite = false;
        }

        for(int i=0;i<cnt-1;i++)
        {
            sData += dataVals[i] + ",";
        }
        sData += dataVals[cnt-1] + "\r\n";

        out << sData;


    }


    cdaq->bTareMode = false;
}


//END SLOTS

QString MainWindow::read_daq_tableWidget_item(int row, int col)
{
    QTableWidgetItem *pTableItem = ui->tableWidget->item(row, col);
    QString cell_text = pTableItem->text();

    return cell_text;
}

void MainWindow::write_gps_tableWidget_item(QString sCellData, int row, int col)
{
    QTableWidgetItem* item1 = new QTableWidgetItem(sCellData);
    item1->setTextAlignment(Qt::AlignCenter);
    //item2->setBackgroundColor(Qt::black);

    ui->tableWidget_2->setItem(row, col, item1);

//    ui->tableWidget_2->setHorizontalHeader(headerView);
//    ui->tableWidget_2->verticalHeader()->setSectionResizeMode(QHeaderView::Stretch);  //this works but we loose alignment...
//    ui->tableWidget_2->resizeColumnsToContents();

}

void MainWindow::write_daq_tableWidget_item(QString sCellData, int row, int col)
{
    QTableWidgetItem* item1 = new QTableWidgetItem(sCellData);
    item1->setTextAlignment(Qt::AlignCenter);
    //item2->setBackgroundColor(Qt::black);

    ui->tableWidget->setItem(row, col, item1);
}


void MainWindow::bubble_daq_tableWidget_rows_down(int numRows, int numCols)
{
    QStringList rowData_list;
    QString sCell_data;

    for(int j=numRows-1; j>0; j--)
    {

        for(int i=0; i<numCols;i++)
        {
            sCell_data = read_daq_tableWidget_item(j, i);

            write_daq_tableWidget_item(sCell_data, j+1, i );
        }

    }
}


void MainWindow::load_gps_header()
{
    //Add Table items here
    QTableWidgetItem* item1 = new QTableWidgetItem(QString("Farm"));
    item1->setTextAlignment(Qt::AlignCenter);
    item1->setBackgroundColor(Qt::gray);

    QTableWidgetItem* item2 = new QTableWidgetItem(QString("Field"));
    item2->setTextAlignment(Qt::AlignCenter);
    item2->setBackgroundColor(Qt::gray);

    QTableWidgetItem* item3 = new QTableWidgetItem(QString("Distance (ft)"));
    item3->setTextAlignment(Qt::AlignCenter);
    item3->setBackgroundColor(Qt::gray);

    QTableWidgetItem* item4 = new QTableWidgetItem(QString("Area (acres)"));
    item4->setTextAlignment(Qt::AlignCenter);
    item4->setBackgroundColor(Qt::gray);


    ui->tableWidget_2->setItem(0,0, item1);
    ui->tableWidget_2->setItem(0,1, item2);
    ui->tableWidget_2->setItem(0,2, item3);
    ui->tableWidget_2->setItem(0,3, item4);


    QFont font;
    font.setBold(true);
    font.setUnderline(true);

    ui->tableWidget_2->item(0, 0)->setFont(font);
    ui->tableWidget_2->item(0, 1)->setFont(font);
    ui->tableWidget_2->item(0, 2)->setFont(font);
    ui->tableWidget_2->item(0, 3)->setFont(font);

}


void MainWindow::load_gps_data(QString sDist, QString sAcres, QString sUTC)
{
    //this code needs to move elsewhere

    //set GPS CHECK-BOX AS CONNECTED
        ui->checkBox->setChecked(true);


        QHeaderView* headerView = new QHeaderView(Qt::Horizontal);
    //    headerView->setResizeMode(QHeaderView::Stretch);

       //QHeaderView* header = ui->tableWidget>horizontalHeader();
        headerView->setSectionResizeMode(QHeaderView::Stretch);

        //test tableWidget
        QStringList labels;
        labels << "" << "CONNECTING" << "";

        //ui->tableWidget->setHorizontalHeader(new myHeaderView());

        ui->tableWidget_2->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding);

        ui->tableWidget_2->setHorizontalHeaderLabels(labels);

        //ui->tableWidget->horizontalHeader()->setResizeMode(0, QHeaderView::Stretch);
        //ui->tableWidget->horizontalHeader()->setResizeContentsPrecision();
        ui->tableWidget_2->verticalHeader()->hide();
        //ui->tableWidget->setShowGrid(false);


        load_gps_header();


        //Add Table items here
        QTableWidgetItem* item1 = new QTableWidgetItem(QString(sFarm));
        item1->setTextAlignment(Qt::AlignCenter);
        //item1->setBackgroundColor(Qt::black);

        QTableWidgetItem* item2 = new QTableWidgetItem(QString(sField));
        item2->setTextAlignment(Qt::AlignCenter);
        //item2->setBackgroundColor(Qt::black);

        QTableWidgetItem* item3 = new QTableWidgetItem(QString(sDist));
        item3->setTextAlignment(Qt::AlignCenter);
        //item2->setBackgroundColor(Qt::black);

        QTableWidgetItem* item4 = new QTableWidgetItem(QString(sAcres));
        item4->setTextAlignment(Qt::AlignCenter);


        ui->tableWidget_2->setItem(1,0, item1);
        ui->tableWidget_2->setItem(1,1, item2);
        ui->tableWidget_2->setItem(1,2, item3);
        ui->tableWidget_2->setItem(1,3, item4);





       // ui->tableWidget->item(0,0)->setBackgroundColor(Qt::red);
       // ui->tableWidget->item(0,1)->setBackgroundColor(Qt::blue);

        //ui->tableWidget->setStyleSheet("QTableView {selection-background-color: red;}");




        //ui->tableView->horizontalHeader()->setSectionResizeMode(QHeaderView::Stretch);

        ui->tableWidget_2->setHorizontalHeader(headerView);
        ui->tableWidget_2->verticalHeader()->setSectionResizeMode(QHeaderView::Stretch);  //this works but we loose alignment...
        ui->tableWidget_2->resizeColumnsToContents();


        //ui->tableWidget->horizontalHeaderItem(0)->setTextAlignment(Qt::AlignLeft);
        ui->tableWidget_2->horizontalHeaderItem(0)->setTextAlignment(Qt::AlignHCenter);
        ui->tableWidget_2->horizontalHeaderItem(1)->setTextAlignment(Qt::AlignHCenter);
        //ui->tableWidget_2->horizontalHeaderItem(2)->setTextAlignment(Qt::AlignHCenter);
        //ui->tableWidget->horizontalHeaderItem(2)->setTextAlignment(Qt::AlignRight);



        /*
            table->setItem(0,0,new QTableWidgetItem("ITEM 1"));
            table->setItem(0,1,new QTableWidgetItem("ITEM 2"));
            table->setItem(0,2,new QTableWidgetItem("ITEM 3"));

            //Color Table Cells Here
            table->item(0,0)->setBackgroundColor(Qt::red);
            table->item(0,1)->setBackgroundColor(Qt::green);
            table->item(0,2)->setBackgroundColor(Qt::blue);
        */
}


void MainWindow::load_daq_header()
{
    //Add Table items here
    QTableWidgetItem* item1 = new QTableWidgetItem(QString("Load #"));
    item1->setTextAlignment(Qt::AlignCenter);
    item1->setBackgroundColor(Qt::gray);

    QTableWidgetItem* item2 = new QTableWidgetItem(QString("Field ID"));
    item2->setTextAlignment(Qt::AlignCenter);
    item2->setBackgroundColor(Qt::gray);

    QTableWidgetItem* item3 = new QTableWidgetItem(QString("Yield (lb/acre)"));
    item3->setTextAlignment(Qt::AlignCenter);
    item3->setBackgroundColor(Qt::gray);

    QTableWidgetItem* item4 = new QTableWidgetItem(QString("Weight (lb)"));
    item4->setTextAlignment(Qt::AlignCenter);
    item4->setBackgroundColor(Qt::gray);

    QTableWidgetItem* item5 = new QTableWidgetItem(QString("Area (acres)"));
    item5->setTextAlignment(Qt::AlignCenter);
    item5->setBackgroundColor(Qt::gray);

    ui->tableWidget->setItem(0,0, item1);
//    ui->tableWidget->setItem(0,1, item2);
    ui->tableWidget->setItem(0,1, item5);
    ui->tableWidget->setItem(0,2, item4);
    ui->tableWidget->setItem(0,3, item3);


    QFont font;
    font.setBold(true);
    font.setUnderline(true);

    ui->tableWidget->item(0, 0)->setFont(font);
    ui->tableWidget->item(0, 1)->setFont(font);
    ui->tableWidget->item(0, 2)->setFont(font);
    ui->tableWidget->item(0, 3)->setFont(font);
//    ui->tableWidget->item(0, 4)->setFont(font);

}


void MainWindow::load_daq_rowData(int rowNum, QString sLoadNum, QString sFieldID, QString sYield, QString sWeight, QString sAcres)
{
    //this code needs to move elsewhere

    //set GPS CHECK-BOX AS CONNECTED
        ui->checkBox->setChecked(true);


        QHeaderView* headerView = new QHeaderView(Qt::Horizontal);
    //    headerView->setResizeMode(QHeaderView::Stretch);

       //QHeaderView* header = ui->tableWidget>horizontalHeader();
        headerView->setSectionResizeMode(QHeaderView::Stretch);

        //test tableWidget
        QStringList labels;
        labels << "" << "CONNECTING" << "";

        //ui->tableWidget->setHorizontalHeader(new myHeaderView());

        ui->tableWidget->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding);

        ui->tableWidget->setHorizontalHeaderLabels(labels);

        //ui->tableWidget->horizontalHeader()->setResizeMode(0, QHeaderView::Stretch);
        //ui->tableWidget->horizontalHeader()->setResizeContentsPrecision();
        ui->tableWidget->verticalHeader()->hide();
        //ui->tableWidget->setShowGrid(false);


        load_daq_header();


        //Add Table items here
        QTableWidgetItem* item1 = new QTableWidgetItem(QString(sLoadNum));
        item1->setTextAlignment(Qt::AlignCenter);
        //item1->setBackgroundColor(Qt::black);

        QTableWidgetItem* item2 = new QTableWidgetItem(QString(sFieldID));
        item2->setTextAlignment(Qt::AlignCenter);
        //item2->setBackgroundColor(Qt::black);

        QTableWidgetItem* item3 = new QTableWidgetItem(QString(sYield));
        item3->setTextAlignment(Qt::AlignCenter);
        //item2->setBackgroundColor(Qt::black);

        QTableWidgetItem* item4 = new QTableWidgetItem(QString(sWeight));
        item4->setTextAlignment(Qt::AlignCenter);
        //item2->setBackgroundColor(Qt::black);

        QTableWidgetItem* item5 = new QTableWidgetItem(QString(sAcres));
        item5->setTextAlignment(Qt::AlignCenter);
        //item2->setBackgroundColor(Qt::black);



        ui->tableWidget->setItem(rowNum,0, item1);
//        ui->tableWidget->setItem(rowNum,1, item2);
        ui->tableWidget->setItem(rowNum,1, item5);
        ui->tableWidget->setItem(rowNum,2, item4);
        ui->tableWidget->setItem(rowNum,3, item3);





       // ui->tableWidget->item(0,0)->setBackgroundColor(Qt::red);
       // ui->tableWidget->item(0,1)->setBackgroundColor(Qt::blue);

        //ui->tableWidget->setStyleSheet("QTableView {selection-background-color: red;}");




        //ui->tableView->horizontalHeader()->setSectionResizeMode(QHeaderView::Stretch);

        ui->tableWidget->setHorizontalHeader(headerView);
        ui->tableWidget->verticalHeader()->setSectionResizeMode(QHeaderView::Stretch);  //this works but we loose alignment...
        ui->tableWidget->resizeColumnsToContents();


        //ui->tableWidget->horizontalHeaderItem(0)->setTextAlignment(Qt::AlignLeft);
        ui->tableWidget->horizontalHeaderItem(0)->setTextAlignment(Qt::AlignHCenter);
        ui->tableWidget->horizontalHeaderItem(1)->setTextAlignment(Qt::AlignHCenter);
        //ui->tableWidget_2->horizontalHeaderItem(2)->setTextAlignment(Qt::AlignHCenter);
        //ui->tableWidget->horizontalHeaderItem(2)->setTextAlignment(Qt::AlignRight);



}



void MainWindow::init_database()
{
return;
    bool ok = test_dataLogger_DB();

    if(!ok)
    {
        create_dataLogger_DB();
    }

    ok = test_dataLogger_DB();

    if(!ok)
    {
        QMessageBox::about(this,tr("Error"), tr("Error loading or creating database"));
    }


}


void MainWindow::load_farmInfo_from_database()
{
    QString tableName2 = "FarmINFO";

    bool ok = csql->testDB_tableExists(tableName2);


    if(ok)
    {
        //now read them back out
        int cnt = csql->getLast_rowID_in_table(tableName2);

        //if(cnt == 1)
        //{

            QStringList sRow;
            int rowID = cnt;
            sRow = csql->getROW_table(tableName2, rowID);

            cnt = sRow.count();

            if(cnt == 6)
            {
                sFarm = sRow[1];
                sClient = sRow[2];
                sField = sRow[3];

                QString sVal = sRow[4];
                int ival = sVal.toInt();
                cgps->num_rows = ival;

                sNum_rows = sVal;
                cgps->sNum_rows = sVal;

                sVal = sRow[5];
                double val = sVal.toDouble();
                cgps->row_width_in = val;
                cgps->row_width_ft = cgps->row_width_in / 12.0;

                sRow_width = sVal;
                cgps->sRow_width_in = sVal;

            }


        //}

    }

}


//read sqlite table to load our settings; such as serial port name "ttySx" ect
void MainWindow::load_settings_from_database()
{


    //START WITH CLEAN SLATE FOR DataLogger DataBase
    //csql->open_sqlite_dataBase();  //this opens default database
    //csql.open_sqlite_dataBase(dbPath);  //this also creates a new database...

    //csql.deleteDB();

    //QString tableName2 = "ModelChannels";
    QString tableName2 = "ENGR_PARMS";

    bool ok = csql->testDB_tableExists(tableName2);


    if(ok)
    {
        //now read them back out
        int cnt = csql->getLast_rowID_in_table(tableName2);

        QStringList sRow;
        for(int i=1;i<=cnt;i++)
        {
            int rowID = i;
            sRow = csql->getROW_table(tableName2, rowID);

            int col_cnt = sRow.count();

            if(col_cnt == 3)
            {
                QString parm_id = sRow[1];
                QString parm_val = sRow[2];

                if(parm_id == "NUM_ROWS")
                {
                    cgps->sNum_rows = parm_val;
                    cgps->num_rows = parm_val.toDouble();
                }
                else if(parm_id == "ROW_WIDTH_IN")
                {
                    cgps->sRow_width_in = parm_val;
                    cgps->row_width_in = parm_val.toDouble();
                    cgps->row_width_ft = cgps->row_width_in / 12.0;
                }
                else if(parm_id == "SLOPE_PRESSURE")
                {
                    double val = parm_val.toDouble();
                    cdaq->slope_press = val;
                }
                else if(parm_id == "INTERCEPT_PRESSURE")
                {
                    double val = parm_val.toDouble();
                    cdaq->intercept_press = val;
                }
                else if(parm_id == "SLOPE_WEIGHT")
                {
                    double val = parm_val.toDouble();
                    cdaq->slope_weight = val;
                }
                else if(parm_id == "INTERCEPT_WEIGHT")
                {
                    double val = parm_val.toDouble();
                    cdaq->intercept_weight = val;
                }
                else if(parm_id == "PWM1_TIMEOUT")
                {
                    double val = parm_val.toDouble();
                    cdaq->pwm1_timeOut = (int)val;
                }
                else if(parm_id == "DUTY_CYCLE_PWM1_FIRST_STAGE")
                {
                    double val = parm_val.toDouble();
                    cdaq->pwm1_firstStage_dutyCycle = (int)val;
                }
                else if(parm_id == "DUTY_CYCLE_PWM1_SECOND_STAGE")
                {
                    double val = parm_val.toDouble();
                    cdaq->pwm1_secondStage_dutyCycle = (int)val;
                }
                else if(parm_id == "WEIGH_OP_REQUEST")
                {
                    cdaq->weighOpRequest =  parm_val;
                }
                else if(parm_id == "DAQ_ALIVE_REQUEST")
                {
                    cdaq->daqAliveRequest = parm_val;
                }
                else if(parm_id == "GPS_PORT_NAME")
                {
                    port_name_gps = parm_val;

                }
                else if(parm_id == "DAQ_PORT_NAME")
                {
                    port_name_daq = parm_val;
                }
                else if(parm_id == "GPS_BAUD_RATE")
                {
                    sPort_baudRate_gps = parm_val;
                    int val = parm_val.toInt();
                    //port_baudRate_gps = val;
                }
                else if(parm_id == "DAQ_BAUD_RATE")
                {
                    sPort_baudRate_daq = parm_val;
                    int val = parm_val.toInt();
                    //port_baudRate_daq = val;

                }
                else if(parm_id == "RADIUS_EARTH_km")
                {
                    QString str = parm_val;
                    double earthRad = parm_val.toDouble();
                    cHav->earth = earthRad;
                    //port_baudRate_daq = val;

                    cHav->test2_distance();

                }



            }

        }
    }


//    csql.closeDB();

}

//see code in cdialog_setup for example
void MainWindow::save_data_to_sql(QStringList colNames, QStringList dataVals)
{
    QString tableName = "DATA";

    int rowID = csql->insertROW_into_table(tableName, colNames, dataVals);


}

void MainWindow::openSerialPort_GPS()
{
    SettingsDialog::Settings p = settings_gps->settings();
    serial_gps->setPortName(port_name_gps);
    serial_gps->setBaudRate(port_baudRate_gps);
    serial_gps->setDataBits(p.dataBits);
    serial_gps->setParity(p.parity);
    serial_gps->setStopBits(p.stopBits);
    serial_gps->setFlowControl(p.flowControl);
    if (serial_gps->open(QIODevice::ReadWrite)) {
            //console->setEnabled(true);
            //console->setLocalEchoEnabled(p.localEchoEnabled);
            //ui->actionConnect->setEnabled(false);
            //ui->actionDisconnect->setEnabled(true);
            //ui->actionConfigure->setEnabled(false);
            ui->statusBar->showMessage(tr("Connected to %1 : %2, %3, %4, %5, %6")
                                       .arg(port_name_gps).arg(port_baudRate_daq).arg(p.stringDataBits)
                                       .arg(p.stringParity).arg(p.stringStopBits).arg(p.stringFlowControl));

            ui->checkBox->setChecked(true);
            init_gps_display();
    } else {
        QMessageBox::critical(this, tr("Error GPS SERIAL CONNECT"), serial_gps->errorString());

        ui->statusBar->showMessage(tr("Open error"));
    }
}


void MainWindow::openSerialPort_DAQ()
{
    SettingsDialog::Settings p = settings_daq->settings();
    serial_daq->setPortName(port_name_daq);
    serial_daq->setBaudRate(port_baudRate_daq);
    serial_daq->setDataBits(p.dataBits);
    serial_daq->setParity(p.parity);
    serial_daq->setStopBits(p.stopBits);
    serial_daq->setFlowControl(p.flowControl);
    if (serial_daq->open(QIODevice::ReadWrite)) {
            //console->setEnabled(true);
            //console->setLocalEchoEnabled(p.localEchoEnabled);
            //ui->actionConnect->setEnabled(false);
            //ui->actionDisconnect->setEnabled(true);
            //ui->actionConfigure->setEnabled(false);
            ui->statusBar->showMessage(tr("Connected to %1 : %2, %3, %4, %5, %6")
                                       .arg(port_name_daq).arg(port_baudRate_daq).arg(p.stringDataBits)
                                       .arg(p.stringParity).arg(p.stringStopBits).arg(p.stringFlowControl));

            ui->checkBox_2->setChecked(true);
            init_daq_display();

    } else {
        QMessageBox::critical(this, tr("Error DAQ SERIAL CONNECT"), serial_daq->errorString());

        ui->statusBar->showMessage(tr("Open error"));
    }
}


void MainWindow::closeSerialPort_GPS()
{
    serial_gps->close();
    //console->setEnabled(false);
    //ui->actionConnect->setEnabled(true);
    //ui->actionDisconnect->setEnabled(false);
    //ui->actionConfigure->setEnabled(true);
    ui->statusBar->showMessage(tr("Disconnected"));
}

void MainWindow::closeSerialPort_DAQ()
{
    serial_daq->close();
    //console->setEnabled(false);
    //ui->actionConnect->setEnabled(true);
    //ui->actionDisconnect->setEnabled(false);
    //ui->actionConfigure->setEnabled(true);
    ui->statusBar->showMessage(tr("Disconnected"));
}



void MainWindow::handleError_GPS(QSerialPort::SerialPortError error)
{
    if (error == QSerialPort::ResourceError) {
        QMessageBox::critical(this, tr("Critical Error"), serial_gps->errorString());
        closeSerialPort_GPS();
    }
}

void MainWindow::handleError_DAQ(QSerialPort::SerialPortError error)
{
    if (error == QSerialPort::ResourceError) {
        QMessageBox::critical(this, tr("Critical Error"), serial_daq->errorString());
        closeSerialPort_DAQ();
    }
}



void MainWindow::readData_daq()
{
    QByteArray data = serial_daq->readAll();

    ui->plainTextEdit->clear();
    ui->plainTextEdit->insertPlainText(data);

    QString data_str = ui->plainTextEdit->toPlainText();

    full_data_str += data_str;


    QStringList strlist = full_data_str.split("\n");

//need to adjust send_command_message() to match!
    QString stopLine = "data101:";

    if(sWindCondition == "CALM")
    {
        stopLine = "data81:";
    }
    else if(sWindCondition == "WINDY")
    {
        stopLine = "data141:";
    }



    int cnt = strlist.count();

    //check last line (or line before last)
    //to see if it's the last expected data line
    //"data23: .... "

    if(rxd_state == 0)
    {
        ui->pushButton_4->setStyleSheet("color: yellow; background-color:blue; border: 1px solid white");
        //ui->pushButton_4->setStyleSheet("color: black; background-color:cyan; border: 1px solid black");
        rxd_state = 1;
    }
    else
    {
        //ui->pushButton_4->setStyleSheet("color: yellow; background-color:blue; border: 1px solid white");
        ui->pushButton_4->setStyleSheet("color: black; background-color:cyan; border: 1px solid black");
        rxd_state = 0;
    }



    for(int i=0;i<cnt;i++)
    {
        QString str = strlist[i];;

        if(str.contains(stopLine))  //if(str.contains("data60:"))
        {
            cdaq->receive_data(strlist);
            //cdaq->parse_message(strlist);

            full_data_str.clear();

            ui->pushButton_4->setStyleSheet("color: yellow; background-color:blue; border: 1px solid white");
            //ui->pushButton_4->setStyleSheet("color: black; background-color:cyan; border: 1px solid black");


            break;
        }
    }



return;


    if(dataBuf.count() < 5)
    {
        return;
    }


    dataBuf.append(data_str);  //append new data to our old data



    //search for 0x55,0x02,0x03 (BASELINE_NED)
    //OR 0x55,0x02,0x05

    for(quint32 i=0;i<(dataBuf.size()-30);i++)
    {

        if(dataBuf.at(i) == 'd')
        {
            dataBuf = dataBuf.mid(i);
            break;
        }


    }


}



void MainWindow::readData_gps()
{
    try
    {
        QByteArray data = serial_gps->readAll();

        if(dataBuf.count() > 60000)
            dataBuf.clear();

        dataBuf.append(data);  //append new data to our old data


        if(dataBuf.count() < 60)
        {
            return;
        }

        QString msg = "";
        QString hdr = "";


        //search for 0x55,0x02,0x03 (BASELINE_NED)
        //OR 0x55,0x02,0x05

        //for(quint32 i=0;i<(dataBuf.size()-60);i++)
        for(quint32 i=0;i<(dataBuf.size()-60);i++)
        {

            //if(dataBuf.at(i) == 0x55)
            if(dataBuf.at(i) == '$')
            {
                dataBuf = dataBuf.mid(i);
                msg = QString(dataBuf);
                dataBuf.clear();
                break;
            }
        }

        int len = msg.length();
        if(len > 60)
        {
            hdr = msg.left(6);
            if(hdr == "$GPRMC")
            {
                //send to gps parser
                cgps->recieve_GPS(msg);
                dataBuf.clear();
            }

        }
        else
        {
            QThread::msleep(500);

        }
    }
    catch(...)
    {
        int x = 5;
        int y = 5;

    }

}
