#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>

#include <QtSerialPort/QSerialPort>
#include <QString>
#include <QStringList>


#include "csql_utils.h"
#include "cdaq_utils.h"
#include "cgps_utils.h"
#include "chaversine.h"


namespace Ui {
class MainWindow;
}

class SettingsDialog;


class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    explicit MainWindow(QWidget *parent = 0);
    ~MainWindow();

    void init_database();

    void load_settings_from_database();

    void load_farmInfo_from_database();


    void run_setup();

    void init_gps_display();  //run after connection
    void init_daq_display();    //run after connection


private slots:

    void load_serialPorts();

    void on_pushButton_clicked();

    void on_pushButton_2_clicked();

    void on_pushButton_3_clicked();

    void on_pushButton_4_clicked();

    void on_pushButton_5_clicked();

//    void save_client_info(QString sFarm, QString sField, QString sClient, QString sNumRows, QString sRowWidth_ft);


    void openSerialPort_GPS();
    void closeSerialPort_GPS();

    void openSerialPort_DAQ();
    void closeSerialPort_DAQ();

    void handleError_GPS(QSerialPort::SerialPortError error);
    void handleError_DAQ(QSerialPort::SerialPortError error);

    void readData_gps();
    void readData_daq();


    void readGPS(QString msg);
    void readDAQ(QString msg);

    QString read_daq_tableWidget_item(int x, int y);
    void bubble_daq_tableWidget_rows_down(int numRows, int numCols);

    void write_gps_tableWidget_item(QString sCellData, int row, int col);
    void write_daq_tableWidget_item(QString sCellData, int row, int col);

    void load_gps_header();
    void load_gps_data(QString sDist, QString sAcres, QString sUTC);

    void load_daq_header();
    void load_daq_rowData(int rowNum, QString sLoadNum, QString sFieldID, QString sYield, QString sWeight, QString sAcres);

    void save_data_to_sql(QStringList colNames, QStringList dataVals);

    void send_command_message();

public:


    QString sFarm="";
    QString sClient="";
    QString sField="";
    QString sNum_rows="6";
    QString sRow_width="40";

    QString sWindCondition = "NORMAL";


    QString sLoad_num = "1";

private:
    Ui::MainWindow *ui;

    QTimer *timer;

    QString csv_filePath;


    int rxd_state = 0;

    bool startUpState = true;

    //QSerialPort *serial;
    QSerialPort *serial_gps;
    QSerialPort *serial_daq;


    QString port_name_daq = "ttyUSB0";
//    QString port_name_gps = "ttyACM0";
    QString port_name_gps = "ttyUSB1";
    QString sPort_baudRate_gps = "9600";
    QString sPort_baudRate_daq = "9600";
    int port_baudRate_gps = 9600;
    int port_baudRate_daq = 115200;


    //SettingsDialog *settings;
    SettingsDialog *settings_gps;
    SettingsDialog *settings_daq;


    quint8 displayCnt = 0;

    QByteArray dataBuf;
    //QByteArray dataBuf_gps;
    //QByteArray dataBuf_daq;

    cSQL_Utils *csql;    //dataBuf.clear();


    cDaq_utils *cdaq;
    cGPS_utils *cgps;
    cHaversine *cHav;

    QString full_data_str;

    bool first_csvWrite = true;

};

#endif // MAINWINDOW_H
